/* 
   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sglauncher.

   sgsched is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgsched is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgsched; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "sg_aplaunch.h"

#define PROC_DIR "/proc"

static Window select_win(Display *dpy, Window top, char *name);

static Window select_win(Display *dpy, Window top, char *name)
{
  Window  root, parent, *child, w = 0;
  int  i, nchild = 0;
  char *win_name;
  int rformat;
  unsigned long rnitem,rafter;
  unsigned char *rdata;
  Atom rtype = None;
  static Atom xa_wm_state = None;

  if (xa_wm_state == None) {
    xa_wm_state = XInternAtom(dpy,"WM_STATE",False);
  }
  if(XFetchName(dpy, top, &win_name) && !strcmp(name, win_name)){
    XGetWindowProperty(dpy,top,xa_wm_state,0,0,
       False,AnyPropertyType,&rtype,&rformat,&rnitem,&rafter,&rdata);
    if (rtype) {
      if (XGetTransientForHint(dpy,top,&w) != 0) {
	return w;
      } else {
	return top;
      }
    }
  }

  if(!XQueryTree(dpy, top, &root, &parent, &child, &nchild))
    return 0;

  for(i = 0; i < nchild; i++){
    w = select_win(dpy, child[i], name);
    if(w) break;
  }

  if(child) XFree((char *)child);

  return w;
}

int SGCheckRunning(char *cmdname)
{
  FILE    *fp;
  DIR     *dir;
  struct dirent *ent;
  char   *p, *q, *r;
  char   cmdline[4096], fname[4096];
  pid_t  pid;
  int    digit;
  int    c;

  dir = opendir(PROC_DIR);
  if(!dir){
    fprintf(stderr, "can't open directory %s!\n", PROC_DIR);
    return -1;
  }

  pid = getpid();
  ent = readdir(dir);

  q = strdup(cmdname);
  r = strchr(q, ' ');
  if(r) *r = '\0';

  while(ent != NULL){
    p = ent->d_name;
    digit = 1;
    while(*p != '\0'){
      if(!(*p >= '0') || !(*p <= '9')){
	digit = 0;
	break;
      }
      p++;
    }
    
    if(digit && (pid != atoi(ent->d_name))){
      sprintf(fname, "%s/%s/cmdline", PROC_DIR, ent->d_name);
      if ((fp = fopen(fname, "r")) != NULL) {
	bzero(cmdline, 4096);
	p = cmdline;
	while((c = fgetc(fp)) != '\0' && c != EOF){
	  *p++ = (char) c;
	}

	fclose(fp);
  #ifdef DEBUG_PRINT
	printf("cmdline: %s:\n", cmdline);	
  #endif
	if(!strcmp(q, cmdline)){
	  free(q);
	  closedir(dir);
	  return 1;
	}
      }
    }
    
    ent = readdir(dir);
  }
  
  free(q);  
  closedir(dir);
  return 0;
}

int SGMapCmd(char *cmdname)
{
  Display             *dpy;
  Window              w;
  char                *cmd, *p;
  Atom                xa_axe_sgws_map;
  XClientMessageEvent ev;

  dpy = XOpenDisplay(NULL);
  if(!dpy) return -1;

  p = cmdname;
  while(*p++ != '\0');
  while(p != cmdname && *p != '/') p--;
  if(*p == '/') cmd = p + 1;
  else          cmd = p;
  
  w = select_win(dpy, RootWindow(dpy, DefaultScreen(dpy)), cmd);
#ifdef DEBUG_PRINT
  printf("Window ID: %x\n", w);
#endif

  if(w){
    XMapRaised(dpy, w);

    xa_axe_sgws_map = XInternAtom(dpy, "_AXE_SGWS_MAP", True);
#ifdef DEBUG_PRINT
    printf("atom: %x\n", xa_axe_sgws_map);
#endif
    ev.type         = ClientMessage;
    ev.window       = DefaultRootWindow(dpy);
    ev.message_type = xa_axe_sgws_map;
    ev.format       = 32;
    ev.data.l[1]    = w;
    XSendEvent(dpy,DefaultRootWindow(dpy), False, 
	       SubstructureNotifyMask,(XEvent *)&ev);
    XFlush(dpy);
    XCloseDisplay(dpy);
    return 1;
  }

  XCloseDisplay(dpy);

  return 0;
}

int SGCheckDinvoke(char *cmdname)
{
  int status;

  status = SGCheckRunning(cmdname);
  if(status > 0){
    SGMapCmd(cmdname);
    return 1;
  }
  
  return status;
}

int SGSingleInvoke(char *cmdname, char *cmdline)
{
  int   status, argc, l, i;
  pid_t pid;
  char  **argv;
  char  *p;

  status = SGCheckRunning(cmdline);

  if(status > 0){
    SGMapCmd(cmdline);
    return 1;
  }else if(status == 0){
    pid = fork();
    if(pid == 0){
      p = cmdline; argc = 0; l = 0;
      while(*p != '\0'){
	if(l > 0 && (*p == ' ')) argc++, l = 0;
	l++, p++;
      }
      
      if(argc > 0){
	argv = calloc(argc + 1, sizeof(char *));
	p = strdup(cmdline);
	i = 0;
	argv[i++] = strtok(p, " ");
	while((argv[i++] = strtok(NULL, " ")) != NULL);

	for (i = 3;i < 32;i++) close(i);
	execvp(argv[0], argv);

	free(p),free(argv);
	_exit(2);
      }else{
#ifdef DEBUG_PRINT
	printf("cmdline: %s\n", cmdline);
#endif
	execlp(cmdline, cmdline, NULL);
	_exit(2);
      }
    }
    return 0;
  }
  return -1;
}
