/* 
   $Id: sgmemo_file.c,v 1.3 2001/12/25 10:41:54 sho Exp $

   Copyright (C) 2001,2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sgmemo.

   sgmemo is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sgmemo is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sgmemo; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <zlib.h>

#include "sgmemo.h"
#include "config.h"

void remove_memo(int num)
{
  DIR  *dirp;
  FILE *fp;
  char *home;
  char dname[4096];
  char fname[4096];

  home = getenv("HOME");

  if(home){
    sprintf(dname, "%s/sgmemo", home);
  }else{
    sprintf(dname, "sgmemo");
  }

  dirp = opendir(dname);

  if(!dirp) mkdir(dname, S_IRWXU);
  else      closedir(dirp);

  sprintf(fname,"%s/%d", dname, num + 1);

  fp = fopen(fname, "r");
  if(fp != NULL){
      fclose(fp);
      unlink(fname);
  }
}

void save_memo(int num, char *str)
{
  DIR  *dirp;
  FILE *fp;
  char *home;
  char dname[4096];
  char fname[4096];

  home = getenv("HOME");

  if(home){
    sprintf(dname, "%s/sgmemo", home);
  }else{
    sprintf(dname, "sgmemo");
  }

  dirp = opendir(dname);

  if(!dirp) mkdir(dname, S_IRWXU);
  else      closedir(dirp);

  sprintf(fname,"%s/%d", dname, num + 1);

  fp = fopen(fname, "w");
  if(fp != NULL){
    if(str && strlen(str) > 0){
      fwrite( str, strlen(str), 1, fp); 
      fclose(fp);
    }else{
      fclose(fp);
      unlink(fname);
    }
  }
}

char *load_memo(int num)
{
  DIR  *dirp;
  FILE *fp;
  char *home;
  char dname[4096];
  char fname[4096];
  char *buf;
  int  size;

  home = getenv("HOME");

  if(home){
    sprintf(dname, "%s/sgmemo", home);
  }else{
    sprintf(dname, "sgmemo");
  }

  dirp = opendir(dname);
  if(!dirp) return NULL;

  closedir(dirp);

  sprintf(fname, "%s/%d", dname, num + 1);
  
  fp = fopen(fname, "r");
  if(!fp) return NULL;

  fseek(fp, 0, SEEK_END);
  size = ftell(fp);

  if(size == 0)    return NULL;

  buf = (char *) malloc(sizeof(char) * (size + 1));
  fseek(fp, 0, SEEK_SET);
  fread(buf, size, 1, fp);
  buf[size] = '\0';

  fclose(fp);  

  return buf;
}

static char     zlib_buf[BUFSIZ];
static z_stream zst;

void init_compress()
{
  zst.zalloc  = Z_NULL;
  zst.zfree   = Z_NULL;
  zst.opaque  = Z_NULL;

  if(deflateInit(&zst, Z_DEFAULT_COMPRESSION) != Z_OK){
    return;
  }

  zst.next_out  = zlib_buf;
  zst.avail_out = BUFSIZ;
}

void compress_write(FILE *fp, void *buf, size_t size)
{
  for(;;){
    if(zst.avail_in == 0){
      zst.next_in  = buf;
      zst.avail_in = size;
      return;
    }
    
    deflate(&zst, Z_NO_FLUSH);
    
    if(zst.avail_out == 0){
      fwrite(zlib_buf, BUFSIZ, 1, fp);
      zst.next_out = zlib_buf;
      zst.avail_out = BUFSIZ;
    }
  }
}

void end_compress(FILE *fp)
{
  int state;
  
  for(;;){
    state = deflate(&zst, Z_FINISH);
    if(state == Z_STREAM_END){
      fwrite(zlib_buf, BUFSIZ - zst.avail_out, 1, fp);
      break;
    }
    
    if(zst.avail_out == 0){
      fwrite(zlib_buf, BUFSIZ, 1, fp);
      zst.next_out  = zlib_buf;
      zst.avail_out = BUFSIZ;
    }
  }
  
  deflateEnd(&zst);
}

void save_stroke(int num, stroke *stk, int n_stroke)
{
  DIR  *dirp;
  FILE *fp;
  char *home;
  char dname[2048];
  char fname[4096];

  home = getenv("HOME");

  if(home){
    sprintf(dname, "%s/sgmemo", home);
  }else{
    sprintf(dname, "sgmemo");
  }

  dirp = opendir(dname);

  if(!dirp) mkdir(dname, S_IRWXU);
  else      closedir(dirp);

  sprintf(fname,"%s/%d.stk", dname, num + 1);

  fp = fopen(fname, "w");
  if(fp != NULL){
    int i, j;
    point tmp;

    if(n_stroke > 0){
      tmp.x = -1;
      tmp.y = -1;
      init_compress();

      for(i = 0; i < n_stroke; i++){
	for(j = 0; j < stk[i].n_sample; j++){
	  compress_write(fp, &stk[i].sample[j], sizeof(point));
	}
	compress_write(fp, &tmp, sizeof(point));
      }
      
      end_compress(fp);
      fclose(fp);
    }else{
      fclose(fp);
      unlink(fname);
    }
  }  
  
}

void
init_uncompress()
{
  zst.zalloc  = Z_NULL;
  zst.zfree   = Z_NULL;
  zst.opaque  = Z_NULL;

  if(inflateInit(&zst) != Z_OK){
    return;
  }

  zst.next_in   = zlib_buf;
  zst.avail_in  = 0;
}

int
end_uncompress(void *buf, size_t size)
{
  int state;

  for(;;){
    state = inflate(&zst, Z_FINISH);
    if(state == Z_STREAM_END){
      break;
    }

    if(zst.avail_out == 0){
      zst.next_out  = buf;
      zst.avail_out = size;
      return 1;
    }
  }

  inflateEnd(&zst);
  return 0;
}

int
uncompress_read(FILE *fp, void *buf, size_t size)
{
  zst.next_out  = buf;
  zst.avail_out = size;
  
  while(zst.avail_out != 0){
    if(zst.avail_in == 0){
      zst.next_in   = zlib_buf;
      zst.avail_in  = read(fileno(fp), zlib_buf, BUFSIZ);

      if(zst.avail_in == 0){
	return 	end_uncompress(buf, size);
      }
    }
    
    inflate(&zst, Z_NO_FLUSH);
  }

  return 1;
}

int load_stroke(int num, stroke *stk)
{
  DIR  *dirp;
  FILE *fp;
  char *home;
  char dname[2048];
  char fname[4096];
  int  n_stroke = 0, n_sample = 0;

  home = getenv("HOME");

  if(home){
    sprintf(dname, "%s/sgmemo", home);
  }else{
    sprintf(dname, "sgmemo");
  }

  dirp = opendir(dname);

  if(!dirp) return 0;
  else      closedir(dirp);
  
  sprintf(fname,"%s/%d.stk", dname, num + 1);

  fp = fopen(fname, "r");

  if(fp != NULL){
    point tmp;
    
    n_stroke = 0, n_sample = 0;

    init_uncompress();

    while(uncompress_read(fp, &tmp, sizeof(point))){
      if(tmp.x == -1 && tmp.y == -1){
	stk[n_stroke].n_sample = n_sample;
	n_sample = 0;
	n_stroke++;
      }else{
	stk[n_stroke].sample[n_sample].x = tmp.x;
	stk[n_stroke].sample[n_sample].y = tmp.y;
	n_sample++;
      }
    }
    
    fclose(fp);
  }

  return n_stroke;
}

void remove_stroke(int num)
{
  DIR  *dirp;
  FILE *fp;
  char *home;
  char dname[4096];
  char fname[4096];

  home = getenv("HOME");

  if(home){
    sprintf(dname, "%s/sgmemo", home);
  }else{
    sprintf(dname, "sgmemo");
  }

  dirp = opendir(dname);

  if(!dirp) mkdir(dname, S_IRWXU);
  else      closedir(dirp);

  sprintf(fname,"%s/%d.stk", dname, num + 1);

  fp = fopen(fname, "r");
  if(fp != NULL){
      fclose(fp);
      unlink(fname);
  }
}
